# Copyright, 2015, by Samuel G. D. Williams. <http://www.codeotaku.com>
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

RSpec.describe Process do
	it "default fork exit status should be 0" do
		pid = fork do
		end
		
		Process.waitpid(pid)
		
		expect($?.exitstatus).to be == 0
	end
	
	it "should fork and return exit status correctly" do
		pid = fork do
			exit(1)
		end
		
		Process.waitpid(pid)
		
		expect($?.exitstatus).to be == 1
	end
	
	# This is currently broken on Rubinius.
	it "should be okay to use fork within a fiber" do
		pid = nil
		
		Fiber.new do
			pid = fork do
				exit(2)
			end
		end.resume
		
		Process.waitpid(pid)
		
		expect($?.exitstatus).to be == 2
	end
end
